const {ipcRenderer, remote} = require('electron');
const $ = require('jquery')

const {createAlert} = require("./sections/utilities/alert_box")

const pageheader = document.getElementById('pageheader')

// APP-LEVEL PERMISSIONS
// emp_pcard_permission, emp_travel_permission, emp_equipment_permission

// SUB-MENU PERMISSIONS
  // HIGH-LEVEL
    // emp_finance_permission, emp_cartegraph_permission, emp_hr_permission
  // LOW-LEVEL
    // emp_position (from list below)
    const emp_position_list = ['Employee', 'Foreman', 'Supervisor', 'Manager', 'Division Manager', 'Deputy Head', 'Department Head']

// OVERRIDE PERMISSIONS
// emp_active == 0: NONE
// emp_admin_permission == 1: ALL

document.getElementById("quit_app").addEventListener("click", ()=>{
  ipcRenderer.invoke('quit_app')
});

console.log('rechecking version')
ipcRenderer.invoke('re-check-version', 'none').then(()=>{
  console.log('version check done')
}).catch(err=>{
  console.log('check for updates error', err.name)
  console.log(err)
  createAlert('Update information could not be loaded')
})

ipcRenderer.on('updateMessage', function(event, text) {
  console.log('update:', text)
  createAlert('info', 'Update Information:' +String(text))
})

let loopcontinue = false
let firstloop = true
async function loopMessage(){
  return setTimeout( async function () {
    if (loopcontinue){
      createAlert('info', 'Working...', 1000)
      firstloop = false
      loopMessage()
    }
  }, firstloop ? 2200 : 1200)
}

//  ______  _    _  _   _   _____  _______  _____  ____   _   _   _____
// |  ____|| |  | || \ | | / ____||__   __||_   _|/ __ \ | \ | | / ____|
// | |__   | |  | ||  \| || |        | |     | | | |  | ||  \| || (___
// |  __|  | |  | || . ` || |        | |     | | | |  | || . ` | \___ \
// | |     | |__| || |\  || |____    | |    _| |_| |__| || |\  | ____) |
// |_|      \____/ |_| \_| \_____|   |_|   |_____|\____/ |_| \_||_____/
document.getElementById('upload_statement').addEventListener('click', async ()=>{
  // createAlert('info', 'Uploading From File: A message will appear when the upload is complete.')
  event.preventDefault()

  await ipcRenderer.invoke('open-file-dialog-csv').then(async selection=>{
    console.log('selection', selection)
    if (selection.canceled){
      createAlert('info', 'Upload Canceled by User')
      return false
    } else {
      createAlert('info', 'Uploading Transactions from Path ' +selection.filePaths[0], 2000)
      loopcontinue = true
      firstloop = true
      loopMessage()
      await ipcRenderer.invoke('upload-new-txn-csv', selection).then(added=>{
        console.log('done')
        loopcontinue = false
        if (added.passed.length > 0){
          console.log('added new statements:', added.passed)
          createAlert('success', 'Added ' +String(added.passed.length) +' new TXNs, Skipped '+String(added.skipped.length) +' duplicates.')
        } else if (added.skipped.length > 0){
          console.log('skipped duplicates:', added.skipped)
          createAlert('info', 'No new TXNs, Skipped '+String(added.skipped.length) +' duplicates.')
        }

        if (added.failed.length > 0){
          console.log('failed on statements:', added.failed)
          failures = 'TXNs Failed - PCARD ENTRY MISSING FOR: \n'
          added.failed.forEach(failure => {
            line = ''+failure["Card Last 4 Digits"]+'; '+failure["CH Full Name"]+' \n'
            if(!failures.includes(line)){
              failures += line
            }
          });
          console.log('failures', failures)
          createAlert(failures)
          let options  = {
            buttons: ["Ok"],
            message: failures
          }
          remote.dialog.showMessageBox(remote.getCurrentWindow(), options)
        }
      }).catch(err=>{
        loopcontinue = false
        createAlert('Statement upload error:' +String(err.name))
      })
    }
  }).catch(err=>{
    createAlert('Statement Select Error:' +String(err.name))
  })
})

const toggle_pcard_manager_review = document.getElementById('toggle_pcard_manager_review')
toggle_pcard_manager_review.addEventListener('click', async()=>{
  event.preventDefault()
  pcard_has_review = await verifStatus()
  console.log('is', pcard_has_review)
  await ipcRenderer.invoke('setPcardRecapHasSupervisorStage', pcard_has_review)
  disp_main.click()
})

async function verifStatus(){
  console.log('verifStatus')
  let Constants = await ipcRenderer.invoke('getConstants')
  console.log('constants', Constants)

  pcard_has_review = Boolean(parseInt(Constants.pcard_recap_manager_review_enabled))
  console.log('managerreview', pcard_has_review)
  
  if (pcard_has_review){
    toggle_pcard_manager_review.innerText = toggle_pcard_manager_review.innerText.replace('Enable', 'Disable')
  } else {
    toggle_pcard_manager_review.innerText = toggle_pcard_manager_review.innerText.replace('Disable', 'Enable')
  }
  return pcard_has_review
}

//  ____   _    _  _______  _______  ____   _   _   _____
// |  _ \ | |  | ||__   __||__   __|/ __ \ | \ | | / ____|
// | |_) || |  | |   | |      | |  | |  | ||  \| || (___
// |  _ < | |  | |   | |      | |  | |  | || . ` | \___ \
// | |_) || |__| |   | |      | |  | |__| || |\  | ____) |
// |____/  \____/    |_|      |_|   \____/ |_| \_||_____/
const disp_main = document.getElementById('disp_main')
$(disp_main).hide()

const disp_pcard = document.getElementById('disp_pcard')
const disp_equipment = document.getElementById('disp_equipment')
const disp_travel = document.getElementById('disp_travel')
const disp_material = document.getElementById('disp_material')
const disp_admin = document.getElementById('disp_admin')

const subpage_main = document.getElementById('subpage_main')
const subpage_pcard = document.getElementById('subpage_pcard')
const subpage_equipment = document.getElementById('subpage_equipment')
const subpage_travel = document.getElementById('subpage_travel')
const subpage_material = document.getElementById('subpage_material')
const subpage_admin = document.getElementById('subpage_admin')

const no_permissions = document.getElementById('no_permissions')

disp_pcard.addEventListener('click', ()=>{
  event.preventDefault()
  $('*[id*="subpage_"]').hide()
  $(subpage_pcard).show()
  $(disp_main).show()
  pageheader.textContent = 'PCARD RECAP APP'
})
disp_equipment.addEventListener('click', ()=>{
  event.preventDefault()
  $('*[id*="subpage_"]').hide()
  $(subpage_equipment).show()
  $(disp_main).show()
  pageheader.textContent = 'EQUIPMENT MANAGEMENT APP'
})
disp_travel.addEventListener('click', ()=>{
  event.preventDefault()
  $('*[id*="subpage_"]').hide()
  $(subpage_travel).show()
  $(disp_main).show()
  pageheader.textContent = 'TRAVEL REQUEST/RECAP APP'
})
disp_material.addEventListener('click', ()=>{
  event.preventDefault()
  $('*[id*="subpage_"]').hide()
  $(subpage_material).show()
  $(disp_main).show()
  pageheader.textContent = 'Material Tracker App'
})
disp_admin.addEventListener('click', ()=>{
  event.preventDefault()
  $('*[id*="subpage_"]').hide()
  $(subpage_admin).show()
  $(disp_main).show()
  pageheader.textContent = 'ADMIN FEATURES'
})

//  _       ______ __      __ ______  _        _____
// | |     |  ____|\ \    / /|  ____|| |      / ____|
// | |     | |__    \ \  / / | |__   | |     | (___
// | |     |  __|    \ \/ /  |  __|  | |      \___ \
// | |____ | |____    \  /   | |____ | |____  ____) |
// |______||______|    \/    |______||______||_____/
let values = window.location.search.split('?')
let stagemode = null
if (values.length >= 1 && values[1]){
  stagemode = values[1]
}
console.log('stagemode:', stagemode)

console.log('reload all constants')
ipcRenderer.invoke('reloadAllConstants', 'none').then(async ()=>{
  console.log('refreshing appdata, please wait')
  const Permissions = await ipcRenderer.invoke('getPermissions')

  await Promise.resolve(Permissions)
  console.log('permissions', Permissions)


  $('*[id*="subpage_"]').hide()
  $('*[name*="_level"]').hide()
  $(no_permissions).show()

  pcard_has_review = await verifStatus()

  // OVERRIDE LEVEL PERMISSIONS
  if (Permissions == null){
    createAlert('UNRECOGNIZED USER. PLEASE CONTACT FINANCE OFFICE TO BE ADDED AS AN APP USER.')
  } else if (Permissions.emp_active == 0){
    createAlert('THIS ACCOUNT HAS BEEN DISABLED BY ADMINISTRATOR. PLEASE CONTACT FINANCE OFFICE TO REENABLE IT.')
  } else if (Permissions.emp_admin_permission == 1){
    // ADMIN LEVEL PERMISSIONS
    $(no_permissions).hide()
    $(subpage_main).show()
    createAlert('info', 'Welcome, Administrator! All Features have been Enabled!')

    $('*[name*="_level"]').show()
    console.log('stagemode', stagemode)
    if (stagemode == 'pcard'){
      disp_pcard.click()
    } else if (stagemode == 'equipment'){
      disp_equipment.click()
    } else if (stagemode == 'travel'){
      disp_travel.click()
    } else if (stagemode == 'material'){
      disp_material.click()
    } else if (stagemode == 'admin'){
      disp_admin.click()
    }
    if (pcard_has_review){
      $('#pcard_division_level').show()
    } else {
      $('#pcard_division_level').hide()
    }
  } else {
    // EMPLOYEE-LEVEL PERMISSIONS
    // ADMIN-LEVEL
    $('*[name*="admin_level"]').hide()

    // HIGH-LEVEL (emp_finance_permission, emp_cartegraph_permission, emp_hr_permission)
    if (Permissions.emp_finance_permission){
      $('*[name*="finance_level"]').show()
    }
    if (Permissions.emp_cartegraph_permission){
      $('*[name*="cartegraph_level"]').show()
    }
    if (Permissions.emp_hr_permission){
      $('*[name*="hr_level"]').show()
    }
    if (Permissions.emp_Equip_submitmode_permission){
      $('*[name*="submit_level"]').show()
    }
    if (Permissions.emp_Equip_approvermode_permission){
      $('*[name*="approve_level"]').show()
        

    }

    // LOW-LEVEL ['Employee', 'Foreman', 'Supervisor', 'Manager', 'Division Manager', 'Deputy Head', 'Department Head']
    
    e_pos = Permissions.emp_position
    if (String(e_pos).includes('Employee')){
      $('*[name*="employee_level"]').show()
    } else if (String(e_pos).includes('Foreman')){
      $('*[name*="employee_level"]').show()
      $('*[name*="foreman_level"]').show()
    } else {
      $('*[name*="employee_level"]').show()
      $('*[name*="foreman_level"]').show()
      $('*[name*="non_employee_level"]').show()

      if (String(e_pos).includes('Supervisor')){
        $('*[name*="supervisor_level"]').show()
      } else if (String(e_pos).includes('Manager')){
        $('*[name*="division_level"]').show()

        if (pcard_has_review){
          $('#pcard_division_level').show()
        } else {
          $('#pcard_division_level').hide()
        }
      } else if (String(e_pos).includes('Deputy')){
        $('*[name*="deputy_level"]').show()
        $('*[name*="department_level"]').show()
      } else if (String(e_pos).includes('Department')){
        $('*[name*="department_level"]').show()
      }

      if (pcard_has_review == true ){
        $('*[name*="supervisor_level"]').show()
         $('*[name*="division_level"]').show()
        $('*[name*="deputy_level"]').show()
        $('*[name*="department_level"]').show()
        //uncomment below line if foreman wants to see the manager review button
        // $('*[name*="foreman_level"]').show() 
      } 
       

    
    }

    // CUSTOM-LEVEL ['PCARD']
    let arg = {
      emp_id:Permissions.emp_id
    }
    ipcRenderer.invoke('load-single-pcard-by-empid', arg).then(result =>{
      console.log('result', result)
      if(result.recordset.length > 0){
        $('*[name*="cardholder_level"]').show()
      } else throw Error
    }).catch(err =>{
      console.log('err', err)
      $('*[name*="cardholder_level"]').hide()
    })

    // APP LEVEL PERMISSIONS
    if (Permissions.emp_pcard_permission) {
      if (Permissions.emp_pcard_admin){
        createAlert('info', 'Welcome, Pcard Admin! All Pcard Features have been Enabled!')
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="pcard_level"]').show()
        $('*[id*="subpage_pcard"]').find('*').show()
        if (pcard_has_review){
          $('#pcard_division_level').show()
        } else {
          $('#pcard_division_level').hide()
        }
      } else {
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="pcard_level"]').show()
      }

      if (!Permissions.emp_budget_permission && !Permissions.emp_travel_permission && !Permissions.emp_equipment_permission || stagemode == 'pcard'){
        // IF ONLY ONE PERMISSION, GO STRAIGHT TO SUBMIT
        if (Permissions.emp_position == 'Employee' && Permissions.emp_finance_permission == 0 && Permissions.emp_cartegraph_permission == 0 && Permissions.emp_hr_permission == 0){
          if (Permissions.emp_pcard_admin){
            $(subpage_main).hide()
            $(subpage_pcard).show()
            $(disp_main).hide()
            pageheader.textContent = 'PCARD RECAP APP'
          } else {
            document.getElementById('pcard_user_submit').click()
          }
        } else {
          $(subpage_main).hide()
          $(subpage_pcard).show()
          $(disp_main).hide()
          pageheader.textContent = 'PCARD RECAP APP'
        }
      }
    }

    if (Permissions.emp_travel_permission) {
      if (Permissions.emp_travel_admin){
        createAlert('info', 'Welcome, Travel Admin! All Travel Features have been Enabled!')
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="travel_level"]').show()
        $('*[id*="subpage_travel"]').find('*').show()
      } else {
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="travel_level"]').show()
      }

      if (!Permissions.emp_budget_permission && !Permissions.emp_pcard_permission && !Permissions.emp_admin_permission && !Permissions.emp_equipment_permission || stagemode == 'travel'){
        $(subpage_main).hide()
        $(subpage_travel).show()
        $(disp_main).hide()
        pageheader.textContent = 'TRAVEL REQUEST/RECAP APP'
      }
    }

    if (Permissions.emp_budget_permission) {
      if (Permissions.emp_budget_admin){
        createAlert('info', 'Welcome, Material Admin! All Budget Features have been Enabled!')
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="material_level"]').show()
        $('*[id*="subpage_material"]').find('*').show()
      } else {
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="material_level"]').show()
      }

      if (!Permissions.emp_pcard_permission && !Permissions.emp_travel_permission && !Permissions.emp_admin_permission && !Permissions.emp_equipment_permission || stagemode == 'material'){
        $(subpage_main).hide()
        $(subpage_material).show()
        $(disp_main).hide()
        pageheader.textContent = 'MATERIAL TRACKER APP'
      }
    }

    if (Permissions.emp_equipment_permission) {
      if (Permissions.emp_equipment_admin){
        createAlert('info', 'Welcome, Equipment Admin! All Equipment Features have been Enabled!')
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="equipadmin_level"]').show()
        // $('*[name*="admin_level"]').show()
        // $('*[name*="pcard_management"]').hide()
        // $('*[name*="org_object_management"]').hide()
        // $('*[name*="recover_completed_entry"]').hide()
        // $('*[name*="toggle_pcard_manager_review"]').hide()
        // $('*[name*="activity_tracker"]').hide()
        $('*[name*="equipment_level"]').show()
        $('*[id*="subpage_equipment"]').find('*').show()
      } else {
        $(no_permissions).hide()
        $(subpage_main).show()
        $('*[name*="equipment_level"]').show()
        // $(disp_main).show()

      }



      if (!Permissions.emp_budget_permission && !Permissions.emp_travel_permission && !Permissions.emp_admin_permission && !Permissions.emp_equipment_permission || stagemode == 'equipment'){
        $(subpage_main).hide()
        $(subpage_equipment).show()
        $(disp_main).show()
        pageheader.textContent = 'EQUIPMENT MANAGEMENT APP'
      }
    }
  }
})


