// Modules to control application life and create native browser window
const {app, BrowserWindow, Menu, MenuItem, ipcMain, dialog} = require('electron')
const path = require('path')
const log = require('electron-log')
const {autoUpdater} = require("electron-updater")
const db = require("./db")
const username = require('username');
const fsPromises = require('fs').promises
const _ = require('lodash');
const unusedFilename = require('unused-filename');
const csv = require('csv-parser')
const fs = require('fs')
const pdf2base64 = require('pdf-to-base64')
const ObjectsToCsv = require('objects-to-csv')

let uname = username().then(result =>{
  uname = result
  log.info('username:', uname)
}).catch(err =>{
  uname = err
  log.info('username error:', uname)
})

autoUpdater.logger = log;
autoUpdater.logger.transports.file.level = 'info';
log.info('App starting...');

let mainWindow

function createWindow () {
  mainWindow = new BrowserWindow({
    icon:path.join(__dirname, '/assets/icons/roadsign/roadsign.ico'),
    width: 1400,
    height: 750,
    webPreferences: {
      nodeIntegration: true,
      preload: path.join(__dirname, 'preload.js')
    },
    autoHideMenuBar: true,
    show: false
  })

  // disp console keybind
  const menu = new Menu()

  menu.append(new MenuItem({
    label: 'Reload',
    accelerator: 'CmdOrCtrl+Shift+R',
    click: ()=>{
      log.info('reload page')
      mainWindow.reload()
    }
  }))
  menu.append(new MenuItem({
    label: 'Console',
    accelerator: 'CmdOrCtrl+Shift+I',
    click: ()=>{
      log.info('show dev tools')
      mainWindow.webContents.openDevTools()
    }
  }))
  Menu.setApplicationMenu(menu)

  mainWindow.on('closed', function () {
    mainWindow = null
  })

  mainWindow.loadFile('index.html')
  mainWindow.maximize()
  mainWindow.once('ready-to-show', () => {
    mainWindow.show()
  })
}

app.on('ready', createWindow)

app.on('window-all-closed', function () {
  // tmpobj.removeCallback();
  app.quit();
})

app.on('activate', function () {
  if (mainWindow === null) createWindow()
})

autoUpdater.channel = 'latest'
autoUpdater.allowDowngrade = false

autoUpdater.logger.transports.file.level = 'silly'
autoUpdater.logger.transports.file.appName = 'private repo'
autoUpdater.autoDownload = true

app.on('ready', async function()  {
  status = await autoUpdater.checkForUpdatesAndNotify()
  if (status == null){
    log.info('dev mode detected: opening side console')
    mainWindow.webContents.openDevTools()
    // mainWindow.loadFile('./sections/utilities/update.html')
  }
});

// //FIXME: uncomment this to auto-update check every 15 minutes. This is NOT enabled because we don't want it to kick a user out of submitting a pcard recap mid-submission for an update.
// setInterval(() => {
//   console.log('Update Check')
//   autoUpdater.checkForUpdatesAndNotify();
// }, 1000 * 60 * 15);

autoUpdater.on('update-available', (info)=>{
  log.info('Update available.');
  mainWindow.webContents.send('updateMessage', 'Update available.');
  mainWindow.loadFile('./sections/utilities/update.html')
})
autoUpdater.on('update-not-available', (info)=>{
  log.info('Update not available.');
})
autoUpdater.on('error', (err)=>{
  log.info('Error in auto-updater. ' + err.toString());
  mainWindow.webContents.send('updateMessage', 'Error in auto-updater. ' + err.toString());
})
autoUpdater.on('download-progress', (progressObj)=>{
  let log_message = 'Downloaded ' + String(parseInt(progressObj.percent)) + '%'
  log.info(log_message);
  mainWindow.webContents.send('updateMessage', log_message);
})
autoUpdater.on('update-downloaded', (info)=>{
  log.info('Update downloaded; The App will now quit to install the update and then reopen.');
  mainWindow.webContents.send('updateMessage', 'Update downloaded; The App will now quit to install the update and then reopen.');
  autoUpdater.quitAndInstall();
});

// IPC COMMUNICATIONS
ipcMain.handle('quit_app', ()=>{
  app.quit()
})

ipcMain.handle('re-check-version', async function (event, arg){
  autoUpdater.checkForUpdatesAndNotify()
  return true
})

ipcMain.handle('form-submission', function (event, arg){
  log.info("generic form submission event:", event)
  log.info("this is the id from the form:", arg)
  return true
})

ipcMain.handle('getExtraUserData', async function (event, arg){
  return await db.getExtraUserData(arg)
})

ipcMain.handle('submitNewTravelForm', async function (event, arg){
  log.info('submit new travel', arg)

  // create event
  return await db.createEvent(arg, uname)
  .then(async result =>{
    // log.info(result)
    arg.travel_event_id = result.recordset[0].event_id
    log.info('event added, id:', arg.travel_event_id)

    // create travel
    let allPass = []
    // log.info(arg.travelers)

    // FIXME: apply this async/await-friendly variant of a for loop wherever necessary
    let i=0
    while (i < arg.travelers.length) {
      let traveler = arg.travelers[i]
      let arg2 = {...arg};
      arg2.travel_emp_id =traveler.emp_id
      arg2.travel_org_key =traveler.org_key
      allPass.push(await db.createTravel(arg2, uname))
      i++
    }

    log.info('all submissions complete!\n')
    return allPass
  })
})

ipcMain.handle('selectionlookup', async (event, arg)=>{
  return await db.selectionlookup(arg)
})

ipcMain.handle('searchlookup', async (event, arg)=>{
  return await db.searchlookup(arg)
})

ipcMain.handle('deletelookup', async (event, arg)=>{
  return await db.deletelookup(arg)
})

ipcMain.handle('reloadAllConstants', async ()=>{
  log.info('reload all')
  try {
    let result = await db.loadequipmenttypes()
    global.equipmenttype = result.recordset;

    result = await db.loadequipmentlocations()
    global.equipmentlocations = result.recordset;

    result = await db.loaddepartments()
    global.departments = result.recordset;

    result = await db.loadequipmentclassifications()
    global.equipmentclassifications = result.recordset;

    result = await db.loadequipmentmanufacturers()
    global.equipmentmanufacturers = result.recordset;

    result = await db.loadequipmentvendors()
    global.equipmentvendors = result.recordset;
  } catch (error) {
    console.log('DB2 CONNECTION FAILED', error)
  }
  try {
    result = await db.loadPermissions(uname)
    global.Permissions = result.recordset[0];
    // log.info('Permissions', global.Permissions)

    result = await db.loadConstants()
    let constants = {}
    result.recordset.forEach(element =>{
      constants[element.const_name] = element.const_value
    });
    global.Constants = constants;
    // log.info('Constants', global.Constants)

    result = await db.loadOrgKeys()
    global.OrgKeys = result.recordset;
    // log.info('OrgKeys', global.OrgKeys)

    result = await db.loadObjCodes()
    global.ObjCodes = result.recordset;
    // log.info('ObjCodes', global.ObjCodes)

    result = await db.loadSupervisors()
    global.Supervisors = result.recordset;
    // log.info('Supervisors', global.Supervisors)
  } catch (error) {
    console.log('DB1 CONNECTION FAILED', error)
  }
  log.info('all globals reloaded')
  return true
})

ipcMain.handle('reloadOrgObj', async ()=>{
  result = await db.loadOrgKeys()
  global.OrgKeys = result.recordset;

  result1 = await db.loadObjCodes()
  global.ObjCodes = result.recordset;

  output = {
    org:result.recordset,
    obj:result1.recordset
  }

  return output
})

// GLOBAL CONSTANTS I/O
ipcMain.handle('getPermissions', async ()=>{
  return global.Permissions
})
ipcMain.handle('getConstants', async ()=>{
  return global.Constants
})
ipcMain.handle('getOrgKeys', async ()=>{
  return global.OrgKeys
})
ipcMain.handle('getObjCodes', async ()=>{
  return global.ObjCodes
})
ipcMain.handle('getSupervisors', async ()=>{
  return global.Supervisors
})
ipcMain.handle('getequipmenttype', async () => {
  return global.equipmenttype
})
ipcMain.handle('getequipmentlocations', async () => {
  return global.equipmentlocations
})

ipcMain.handle('getdepartments', async () => {
  return global.departments
})

ipcMain.handle('getequipmentclassifications', async () => {
  return global.equipmentclassifications
})

ipcMain.handle('getequipmentmanufacturers', async () => {
  return global.equipmentmanufacturers
})

ipcMain.handle('getequipmentvendors', async () => {
  return global.equipmentvendors
})

ipcMain.handle('getUname', async ()=>{
  return uname
})

// FILE OPS

// FIXME: update travel request to support new file-dialog return type
ipcMain.handle('open-file-dialog-pdf', async ()=>{
  return await dialog.showOpenDialog(
    mainWindow,
    {
      properties: [
        'openFile'
      ],
      message: 'Select PDF of Documentation',
      filters: [
        { name: 'PDF', extensions: ['pdf'] }
      ]
    }
  )
})

ipcMain.handle('open-file-dialog-csv', async ()=>{
  selection = await dialog.showOpenDialog(
    mainWindow,
    {
      properties: [
        'openFile'
      ],
      message: 'Select CSV to Import Data from',
      filters: [
        { name: 'CSV', extensions: ['csv', 'txt'] }
      ]
    }
  )

  if (selection.canceled){
    log.info('canceled')
    return false
  } else {
    log.info('selection', selection.filePaths[0])
    return selection
  }
})

ipcMain.handle('upload-new-txn-csv', async (event, selection)=>{
  if (selection.canceled){
    log.info('canceled')
    return false
  } else {
    log.info('selection', selection.filePaths[0])

    const results = []

    output = {
      path : selection.filePaths[0]
    }

    return new Promise((resolve, reject)=>{
      fs.createReadStream(selection.filePaths[0])
      .pipe(csv())
      .on('data', (data)=> results.push(data))
      .on('end', async()=>{
        log.info('done');
        listvals = results
        const skipped = []
        const failed = []
        const passed = []

        for (let i = 0; i < listvals.length; i++) {
          const entry = listvals[i];
          await db.pcardAddTransaction(entry).then(()=>{
            passed.push(entry)
          }).catch(err=>{
            if (String(err.stack).includes('column does not allow nulls')){
              failed.push(entry)
            } else if (String(err.stack).includes('Violation of PRIMARY KEY constraint')){
              skipped.push(entry)
            } else {
              log.info(err)
              failed.push(entry)
            }
          })
        }

        arg = {
          log_linked_table:'filesys',
          log_type:'INSERT',
          log_desc:'UPLOAD NEW TXNS: ' +String(passed.length) +' added, ' +String(skipped.length) +' skipped, ' +String(failed.length) +' failed',
          log_uname:uname
        }

        if (passed.length > 0){
          const csv = new ObjectsToCsv(listvals)
          const csvstring = await csv.toString('base64')

          arg.filesys_linked_table='pcard_recap'
          arg.filesys_linked_id='DAILY_TXN_UPLOAD'
          arg.filesys_name='DAILY_TXN_UPLOAD'
          arg.filesys_content=csvstring

          result = await db.uploadFile(arg)
          console.log('saved file to', result.recordset[0].filesys_id)
          arg.log_linked_id = result.recordset[0].filesys_id
          await db.dbLogQuery(arg)
        }

        return resolve({passed:passed, skipped:skipped, failed:failed})
      })
      .on('error', (error)=>{
        log.info('error')
        return reject(error)
      })
    })
  }
})


async function loadAllCartegraphUsers(){
  await db.loadAllCartegraphUsers().then(async result=>{
    console.log('CARTEGRAPH USERS')
    outlist = result.recordset

    const csv = new ObjectsToCsv(outlist)
    const csvstring = await csv.toString()
    const options = {
      title: 'Save data to CSV',
      filters: [
        { name: 'CSV', extensions: ['csv'] }
      ],
      buttonLabel:'Save CSV (Excel)'
    }

    outpath = dialog.showSaveDialogSync(mainWindow, options)
    if (!outpath){
      return -1
    } else {
      // return await csvstring.toDisk(outpath)
      return await fsPromises.writeFile(outpath, csvstring)
    }
  }).catch(err=>{
    console.log('CARTEGRPH ERROR', err)
    return false
  })
}
// loadAllCartegraphUsers()

async function saveDuplicatableDocument(inpath, tempoutpath) {
  dirpath = tempoutpath.split('\\').slice(0,-1).join('\\')
  fs.mkdirSync(dirpath, { recursive: true })
  outpath = await unusedFilename(tempoutpath)
  fs.copyFileSync(inpath, outpath)
  return outpath
}

ipcMain.handle('save-file-to-filesys', async (event, arg)=>{
  arg.filesys_content = await pdf2base64(arg.path)
  return await db.uploadFile(arg)
})

ipcMain.handle('load-file-list-filesys', async(event, arg)=>{
  return await db.loadFileList(arg)
})

ipcMain.handle('loadAllFileContents', async(event, arg)=>{
  return await db.loadAllFileContents(arg)
})

// ipcMain.handle('loadselectedlist', async(event, arg)=>{
//   return await db.loadselectedlist(arg)
// })

ipcMain.handle('loadequipmentlist', async(event, arg)=>{
  return await db.loadequipmentlist(arg)
})

ipcMain.handle('load-file-filesys', async(event, arg)=>{
  return await db.loadFile(arg)
})

ipcMain.handle('csv-save-dialog', async(event, listvals)=>{
  const csv = new ObjectsToCsv(listvals)
  const csvstring = await csv.toString()
  const options = {
    title: 'Save data to CSV',
    filters: [
      { name: 'CSV', extensions: ['csv'] }
    ],
    buttonLabel:'Save CSV (Excel)'
  }

  outpath = dialog.showSaveDialogSync(mainWindow, options)
  if (!outpath){
    return -1
  } else {
    // return await csvstring.toDisk(outpath)
    return await fsPromises.writeFile(outpath, csvstring)
  }
})

// generic travel loaders
ipcMain.handle('load-travel-recap', async (event, arg)=>{
  return await db.loadTravelRecap(arg)
})

// generic PCARD loaders
ipcMain.handle('load-pcard-recap', async function (event, arg){
  return await db.loadPcardRecap(arg)
})

let startemp = {}
ipcMain.handle('submitUserLookupForm', async function (event, arg){
  return await db.userLookup(arg)
})

ipcMain.handle('load-all-employees', async ()=>{
  return await db.loadAllUsers()
})

ipcMain.handle('load-single-employee', async (event, emp_id)=>{
  arg = {emp_id: emp_id}
  result = await db.userLookup(arg)

  if (result && result && result.recordset[0]){
    startemp =  result.recordset[0]
  } else {
    startemp = {}
  }
  return result
})

ipcMain.handle('load-single-pcard', async (event, cardnum)=>{
  if(cardnum.length == 16){
    log.info('load single pcard', '12', cardnum.slice(0,12), '4', cardnum.slice(-4))
    arg = {
      pcard_first_12: cardnum.slice(0,12),
      pcard_last_4: cardnum.slice(-4)
    }
  } else {
    log.info('load all pcards', '4', cardnum.slice(-4))
    arg = {
      pcard_last_4: cardnum.slice(-4)
    }
  }
  result = await db.pcardLookup(arg)

  if (result && result && result.recordset[0]){
    startemp =  result.recordset[0]
  } else {
    startemp = {}
  }
  return result
})

ipcMain.handle('load-single-pcard-by-empid', async (event, arg)=>{
  return await db.pcardLookupEmpid(arg)
})

ipcMain.handle('modify-single-employee', async (event, arg)=>{
  if (_.isMatch(startemp, arg)) {
    return arg
  } else {
    await db.userModify(arg, uname)
    return arg
  }
})

ipcMain.handle('load-all-pcards', async ()=>{
  return await db.loadAllPcards()
})

const isSubset = (aSubset, aSuperset)=> (
  _.every(aSubset, (val, key)=> _.isEqual(val, aSuperset[key]))
)

ipcMain.handle('modify-single-pcard', async (event, arg)=>{
  if (isSubset(arg, startemp)) {
    return arg
  } else {
    return await db.pcardModify(arg, uname)
  }
})

ipcMain.handle('modify-single-pcard-recap', async (event, arg)=>{
  arg.uname = uname
  return db.pcardRecapModify(arg, uname).then(result=>{
    console.log('result', result)
    return result
  }).catch(err=>{
    return err
  })
})

ipcMain.handle('modify-single-travel-recap', async (event, arg)=>{
  arg.uname = uname
  return db.travelModify(arg, uname).then(result=>{
    console.log('result', result)
    return result
  }).catch(err=>{
    return err
  })
})

ipcMain.handle('modifyaddequipmentform', async (event, arg) => {
  arg.uname = uname
  return db.modifyaddequipmentform(arg, uname).then(result=>{
    console.log('result', result)
    return result
  }).catch(err=>{
    return err
  })
})

ipcMain.handle('modify-selected-equipment-recap', async (event, arg)=>{
  arg.uname = uname
  return db.selectedequipmentmodify(arg, uname).then(result=>{
    console.log('result', result)
    return result
  }).catch(err=>{
    return err
  })
})
const {makePDF, makeTravelPDF} = require('./sections/utilities/reportgen')
ipcMain.handle('generate-recap-pdf', async function (event, arg) {
  // FIXME: THIS DOESN"T WORK FOR NO UNDERSTOOD REASON; MAKEPDF DOESN"T WORK WHEN CALLED BY MAIN, BUT WORKS WHEN CALLED BY WEBPAGE JS
  // FOR NOW, DO NOT INVOKE THIS FUNCTION
  return await makePDF(arg)
})

ipcMain.handle('setPcardRecapHasSupervisorStage', async (event, pcard_target_review)=>{
  return await db.setPcardRecapHasSupervisorStage(pcard_target_review, uname)
})

ipcMain.handle('load-pcard-recap-subitems', async (event, arg)=>{
  return await db.pcardRecapLoadSubitems(arg)
})

ipcMain.handle('update-pcard-recap-subitem', async (event, arg)=>{
  return await db.pcardRecapAddUpdateSubitem(arg)
})

ipcMain.handle('delete-pcard-recap-subitem', async (event, arg)=>{
  return await db.pcardRecapDeleteSubitem(arg)
})

ipcMain.handle('submitaddequipmentform', async (event, arg) => {
  result = await db.submitaddequipmentform(arg)
  console.log('inserted id', result.recordset[0].Equipmentformid)

  arg2 = {
    log_linked_table:'modifyequipment',
    log_linked_id:result.recordset[0].Equipmentformid,
    log_type:'INSERT',
    log_desc:'User Submit New Equipment Request',
    log_uname:uname,
    log_note:arg.notes
  }
  return await db.dbLogQuery(arg2)
})

ipcMain.handle('org_load', async(event, arg)=>{
  return await db.org_load(arg)
})

ipcMain.handle('obj_load', async(event, arg)=>{
  return await db.obj_load(arg)
})

ipcMain.handle('org_add', async(event, arg)=>{
  return await db.org_add(arg, uname)
})

ipcMain.handle('obj_add', async(event, arg)=>{
  return await db.obj_add(arg, uname)
})

ipcMain.handle('org_edit', async(event, arg)=>{
  return await db.org_edit(arg, uname)
})

ipcMain.handle('obj_edit', async(event, arg)=>{
  return await db.obj_edit(arg, uname)
})

log.info('Main Loaded')