const {ipcRenderer} = require('electron')
const $ = require('jquery')

const {createAlert} = require("../utilities/alert_box")
const {makeSortable} = require('../utilities/sortable')
makeSortable()

document.getElementById("quit_app").addEventListener("click", ()=>{
  ipcRenderer.invoke('quit_app')
});

let values = window.location.search.split('?')
let stagemode = null
if (values.length >= 1 && values[1]){
  stagemode = values[1]
}
console.log('stagemode', stagemode)
baseurl = '../../index.html'
if (stagemode == null){
  targeturl = baseurl + '?admin'
} else {
  targeturl = baseurl + '?'+String(stagemode)
}
document.getElementById('goto_home').href = targeturl

// modal closers:
let orgobj_Modal = document.getElementById('orgobj_Modal')
let span = [...document.getElementsByClassName("close")];
span.forEach(button =>{
  button.onclick = function() {
    $(orgobj_Modal).hide()
  }
});

window.onclick = function(event) {
  if (event.target == orgobj_Modal) {
    $(orgobj_Modal).hide()
  }
}

const org_list = document.getElementById('org_list')
const obj_list = document.getElementById('obj_list')

let selection = null
async function reloadAllOrgObjs(){
  selection = null
  let trs = document.getElementsByClassName('singleselect')
  for(let i = trs.length-1; i >= 0; i--){
    trs[i].remove()
  }

  await ipcRenderer.invoke('reloadOrgObj').then(result=>{
    console.log('org_list:', result.org)
    for (let i = 0; i < result.org.length; i++) {
      let row = org_list.insertRow()
      row.className = 'singleselect'
      hiddencell = row.insertCell(0)
      hiddencell.textContent = JSON.stringify(result.org[i])
      $(hiddencell).hide()

      row.insertCell(1).textContent = result.org[i].org_key
      row.insertCell(2).textContent = result.org[i].org_desc
      row.insertCell(3).textContent = (result.org[i].org_enabled?'\u2714':'\u274C')
    }

    console.log('obj_list:', result.obj)
    for (let i = 0; i < result.obj.length; i++) {
      let row = obj_list.insertRow()
      row.className = 'singleselect'
      hiddencell = row.insertCell(0)
      hiddencell.textContent = JSON.stringify(result.obj[i])
      $(hiddencell).hide()

      row.insertCell(1).textContent = result.obj[i].obj_code
      row.insertCell(2).textContent = result.obj[i].obj_desc
      row.insertCell(3).textContent = (result.obj[i].obj_enabled?'\u2714':'\u274C')
    }
    makeSelectable()
  }).catch( err =>{
    console.log(err)
  })
}

function makeSelectable(){
  let trs = document.querySelectorAll("tr");
  for(let i = 0; i < trs.length; i++){
    console.log('table row class:', trs[i].className)
    if (trs[i].className.includes('multiselect')){
      trs[i].addEventListener("click", function(){
        // console.log('selection set')
        let cn = this.className, thc = " selected", start_idx = cn.indexOf(thc);
        if(start_idx == -1) cn += thc;
        else cn = cn.replace(thc,"");
        this.className = cn;
      })
    } else if (trs[i].className.includes('singleselect')){
      trs[i].addEventListener("click", function(){
        // console.log('selection set')
        for(let i = 0; i < trs.length; i++){
          let cn = trs[i].className, thc = " selected", start_idx = cn.indexOf(thc);
          if(start_idx != -1 && trs[i].className.includes('singleselect')) {
            cn = cn.replace(thc,"");
            trs[i].className = cn;
          }
        }
        let cn = this.className, thc = " selected", start_idx = cn.indexOf(thc);
        if(start_idx == -1) cn += thc;
        this.className = cn;
        selection = JSON.parse(this.cells[0].textContent)
        console.log('selection set', selection)
      })
    }
  }
}

// button ops
let addmode = false
document.getElementById('org_toggle').addEventListener('click', async()=>{
  event.preventDefault()
  if (selection && selection.org_key){
    console.log('toggle org_key')
    arg = {
      org_key:selection.org_key,
      org_desc:selection.org_desc,
      org_enabled:(selection.org_enabled?0:1)
    }
    await ipcRenderer.invoke('org_edit', arg).then(result=>{
      console.log('update result', result)
      reloadAllOrgObjs()
    }).catch(err=>{
      createAlert('update issue: '+err.toString())
      reloadAllOrgObjs()
    })
  } else {
    createAlert('must select an org key to toggle')
  }
})
document.getElementById('obj_toggle').addEventListener('click', async()=>{
  event.preventDefault()
  if (selection && selection.obj_code){
    console.log('toggle obj_code')
    arg = {
      obj_code:selection.obj_code,
      obj_desc:selection.obj_desc,
      obj_enabled:(selection.obj_enabled?0:1)
    }
    await ipcRenderer.invoke('obj_edit', arg).then(result=>{
      console.log('update result', result)
      reloadAllOrgObjs()
    }).catch(err=>{
      createAlert('update issue: '+err.toString())
      reloadAllOrgObjs()
    })
  } else {
    createAlert('must select an obj code to toggle')
  }
})
document.getElementById('org_new').addEventListener('click', ()=>{
  event.preventDefault()
  addmode = true
  console.log('add new org key')
  dispOrgobjModal({mode:'org'})
})
document.getElementById('obj_new').addEventListener('click', ()=>{
  event.preventDefault()
  addmode = true
  console.log('add new object code')
  dispOrgobjModal({mode:'obj'})
})
document.getElementById('org_edit').addEventListener('click', ()=>{
  event.preventDefault()
  addmode = false
  if (selection && selection.org_key){
    console.log('edit org key')
    dispOrgobjModal(selection)
  } else {
    createAlert('must select an org key to edit')
  }
})
document.getElementById('obj_edit').addEventListener('click', ()=>{
  event.preventDefault()
  addmode = false
  if (selection && selection.obj_code){
    console.log('edit obj code')
    dispOrgobjModal(selection)
  } else {
    createAlert('must select an obj code to edit')
  }
})

let mode = null
async function dispOrgobjModal(target){
  console.log('load target', target)

  // reset values
  $('#orgobj_id').val('')
  $('#orgobj_desc').val('')
  $('#orgobj_enabled').val('')

  // FIXME: NEVER DO THE LINE BELOW: IT RESETS THE VALUE WITHIN SELECT OPTION BOXES, MAKING THEIT SELECTION NOT ABLE TO BE CORRECTLY SET/GET
  // $(orgobj_Modal).find('*').val('')

  let result = null

  // load values
  if (target && target.org_key){
    console.log('modify org_key', target.org_key)
    mode = 'org'
    arg = {
      org_key:target.org_key
    }
    result = await ipcRenderer.invoke('org_load', arg)
  } else if (target && target.obj_code){
    console.log('modify obj_code', target.obj_code)
    arg = {
      obj_code:target.obj_code
    }
    mode = 'obj'
    result = await ipcRenderer.invoke('obj_load', arg)
  } else if (target.mode == 'org'){
    console.log('new org_key')
    mode = 'org'
  } else if (target.mode == 'obj') {
    console.log('new obj_code')
    mode = 'obj'
  } else {
    createAlert('failed to load modal; invalid conditions')
    return
  }

  // change title
  if (mode == 'org'){
    document.getElementById('orgobj_title').textContent = 'Org Key Information:'
    document.getElementById('orgobj_id_label').textContent = 'Org Key:\xa0'
  } else {
    document.getElementById('orgobj_title').textContent = 'Obj Code Information:'
    document.getElementById('orgobj_id_label').textContent = 'Obj Code:\xa0'
  }

  // set values
  if (result && result.recordset && result.recordset[0]){
    if (result.recordset[0].org_key){
      $('#orgobj_id').val(result.recordset[0].org_key)
      $('#orgobj_id').prop('required',false);
      $('#orgobj_id').prop('disabled',true);
      $('#orgobj_desc').val(result.recordset[0].org_desc)
      $('#orgobj_enabled').val(result.recordset[0].org_enabled)
    } else if (result.recordset[0].obj_code) {
      $('#orgobj_id').val(result.recordset[0].obj_code)
      $('#orgobj_id').prop('required',false);
      $('#orgobj_id').prop('disabled',true);
      $('#orgobj_desc').val(result.recordset[0].obj_desc)
      $('#orgobj_enabled').val(result.recordset[0].obj_enabled)
    }
  } else {
    $('#orgobj_id').prop('disabled',false);
    $('#orgobj_id').prop('required',true);
  }

  // disp modal
  $(orgobj_Modal).show();
}

document.getElementById('orgobj_cancel').addEventListener('click', ()=>{
  $(orgobj_Modal).hide()
  // TODO: decide if this is redundant or helpful
  reloadAllOrgObjs()
})

document.getElementById('orgobj_confirm').addEventListener('click', ()=>{
  arg = {}

  if (document.getElementById('modifyOrgObjForm').checkValidity()) {
    if (mode == 'org'){
      arg.org_key = document.getElementById('orgobj_id').value
      arg.org_desc = document.getElementById('orgobj_desc').value
      arg.org_enabled = document.getElementById('orgobj_enabled').value

      ipcRenderer.invoke((addmode?'org_add':'org_edit'), arg).then(result=>{
        createAlert('info', (addmode?'org_add':'org_edit')+' succeeded!')
        $(orgobj_Modal).hide()
        reloadAllOrgObjs()
      }).catch(err=>{
        createAlert('Failed'+err.toString())
      })
    } else {
      arg.obj_code = document.getElementById('orgobj_id').value
      arg.obj_desc = document.getElementById('orgobj_desc').value
      arg.obj_enabled = document.getElementById('orgobj_enabled').value

      ipcRenderer.invoke((addmode?'obj_add':'obj_edit'), arg).then(result=>{
        createAlert('info', (addmode?'obj_add':'obj_edit')+' succeeded!')
        $(orgobj_Modal).hide()
        reloadAllOrgObjs()
      }).catch(err=>{
        createAlert('Failed'+err.toString())
      })
    }
  } else {
    createAlert('danger', 'Form invalid; Update impossible')
    document.getElementById('modifyOrgObjForm').reportValidity()
  }
})

// on load everything else
reloadAllOrgObjs()