const {ipcRenderer} = require('electron')
const $ = require('jquery')

const {createAlert} = require("../utilities/alert_box")
const {makeSortable} = require('../utilities/sortable')
makeSortable()

const pcard_default_org_key = document.getElementById('pcard_default_org_key')
const pcard_default_obj_code = document.getElementById('pcard_default_obj_code')

document.getElementById("quit_app").addEventListener("click", ()=>{
  ipcRenderer.invoke('quit_app')
});

let values = window.location.search.split('?')
let stagemode = null
if (values.length >= 1 && values[1]){
  stagemode = values[1]
}
console.log('stagemode', stagemode)
baseurl = '../../index.html'
if (stagemode == null){
  targeturl = baseurl + '?admin'
} else {
  targeturl = baseurl + '?'+String(stagemode)
}
document.getElementById('goto_home').href = targeturl

document.getElementById("exportlist_excel").addEventListener('click', async()=>{
  event.preventDefault()

  await ipcRenderer.invoke('load-all-pcards').then(async result=>{
    listvals = result.recordset

    // FILTER VALUES
    const keys = Object.keys(listvals[0])
    console.log('keys', keys)
    for (let i = 0; i < listvals.length; i++) {
      for (let j = 0; j < keys.length; j++){
        if (String(keys[j]).startsWith('emp_')){
          delete(listvals[i][String(keys[j])])
        } else if (listvals[i][String(keys[j])] && listvals[i][String(keys[j])].toUTCString){
          listvals[i][String(keys[j])] = listvals[i][String(keys[j])].toUTCString().substring(4,16)
        }
      }
    }

    // SAVE FILE
    await ipcRenderer.invoke('csv-save-dialog', listvals).then(results=>{
      if (results == -1) {
        createAlert('info', 'Export File Cancelled')
      } else {
        createAlert('success', 'CSV Exported!')
      }
    }).catch(err=>{
      createAlert('error', 'File Save Error: '+err.toString())
    })
  }).catch(error=>{
    createAlert('warning', 'Export Excel Error ' +error.toString())
  })
})

OrgKeys = ipcRenderer.invoke('getOrgKeys').then(result =>{
  OrgKeys = result
  console.log('pcard_default_org_key:', OrgKeys)
  for (let i = 0; i < OrgKeys.length; i++) {
    if (OrgKeys[i].org_enabled == 1){
      let name = OrgKeys[i].org_key +': '+ OrgKeys[i].org_desc
      let id = OrgKeys[i].org_key
      option = document.createElement('option')
      option.value = id
      option.text = name
      pcard_default_org_key.add(option)
    }
  }
})

ObjCodes = ipcRenderer.invoke('getObjCodes').then(result =>{
  ObjCodes = result
  console.log('pcard_default_obj_code:', ObjCodes)
  for (let i = 0; i < ObjCodes.length; i++) {
    if (ObjCodes[i].obj_enabled == 1){
      let name = ObjCodes[i].obj_code +': '+ ObjCodes[i].obj_desc
      let id = ObjCodes[i].obj_code
      option = document.createElement('option')
      option.value = id
      option.text = name
      pcard_default_obj_code.add(option)
    }
  }
})

let selection = null

function makeSelectable(){
  let trs = document.querySelectorAll("tr");
  for(let i = 0; i < trs.length; i++){
    console.log('table row class:', trs[i].className)
    if (trs[i].className.includes('multiselect')){
      trs[i].addEventListener("click", function(){
        // console.log('selection set')
        let cn = this.className, thc = " selected", start_idx = cn.indexOf(thc);
        if(start_idx == -1) cn += thc;
        else cn = cn.replace(thc,"");
        this.className = cn;
      })
    } else if (trs[i].className.includes('singleselect')){
      trs[i].addEventListener("click", function(){
        // console.log('selection set')
        for(let i = 0; i < trs.length; i++){
          let cn = trs[i].className, thc = " selected", start_idx = cn.indexOf(thc);
          if(start_idx != -1 && trs[i].className.includes('singleselect')) {
            cn = cn.replace(thc,"");
            trs[i].className = cn;
          }
        }
        let cn = this.className, thc = " selected", start_idx = cn.indexOf(thc);
        if(start_idx == -1) cn += thc;
        this.className = cn;
        selection = String(this.cells[0].textContent)+String(this.cells[1].textContent)
        console.log('selection set', selection)
      })
    }
  }
}

const table = document.getElementById('pcardlist')

async function reloadAllUsers(){
  selection = null
  let trs = document.getElementsByClassName('singleselect')
  for(let i = trs.length-1; i >= 0; i--){
    trs[i].remove()
  }

  await ipcRenderer.invoke('load-all-pcards').then( result =>{
    console.log('pcard list', result.recordset)

    for(let i = 0; i < result.recordset.length; i++){
      let row = table.insertRow()
      row.className = 'singleselect'
      row.insertCell(0).textContent = String(result.recordset[i].pcard_first_12)

      row.insertCell(1).textContent = String(result.recordset[i].pcard_last_4)

      row.insertCell(2).textContent = String(result.recordset[i].pcard_enabled?"Yes": "No")

      row.insertCell(3).textContent = result.recordset[i].emp_lname

      row.insertCell(4).textContent = result.recordset[i].emp_fname

      if (result.recordset[i].pcard_issue_date){
        let d1 = new Date(result.recordset[i].pcard_issue_date)
        row.insertCell(5).textContent = (d1.toUTCString() +', ' +d1.getUTCFullYear()).substring(4, 16).padEnd(15)
      } else row.insertCell(5).textContent = 'N/A'

      if (result.recordset[i].pcard_expire_date){
        let d2 = new Date(result.recordset[i].pcard_expire_date)
        row.insertCell(6).textContent = (d2.toUTCString() +', ' +d2.getUTCFullYear()).substring(4, 16).padEnd(15)
      } else row.insertCell(6).textContent = 'N/A'

      if (result.recordset[i].pcard_limit){
        row.insertCell(7).textContent = ''+
          String(parseFloat(result.recordset[i].pcard_limit)).split('.')[0] +'.' +
          (String(parseFloat(result.recordset[i].pcard_limit+0.000001)).split('.')[1]+'').substring(0,2)
      } else row.insertCell(7).textContent = 'Unknown'
    }
    makeSelectable()
  }).catch( err =>{
    console.log(err)
  })
}

document.getElementById('emp_id').addEventListener('input', event=>{
  if (event.target.validity.valid){
    console.log('emp_lookup_by_id', event.target.value)
    const arg = {
      emp_id: event.target.value
    }
    ipcRenderer.invoke('submitUserLookupForm', arg).then(result=>{
      event.target.setCustomValidity("");
      console.log('loaded employee', result)
      document.getElementById('emp_uname').value = result.recordset[0].emp_uname
      document.getElementById('emp_lname').value = result.recordset[0].emp_lname
      document.getElementById('emp_fname').value = result.recordset[0].emp_fname

    }).catch(err=>{
      createAlert('Employee Not Found. If ID represents an employee who should be in the system, please contact Application Administrator to add them.')
      document.getElementById('emp_uname').value = ''
      document.getElementById('emp_lname').value = ''
      document.getElementById('emp_fname').value = ''
      event.target.setCustomValidity("Invalid field.");
    })
  }
  event.target.setCustomValidity("");
  document.getElementById('emp_uname').value = ''
  document.getElementById('emp_lname').value = ''
  document.getElementById('emp_fname').value = ''
})

document.getElementById('lookup_pcard').addEventListener('click', ()=>{
  event.preventDefault()

  target = String(document.getElementById('lookup_pcard_first_12').value)+
  String(document.getElementById('lookup_pcard_last_4').value)
  console.log('target', target)

  if(document.getElementById('lookupPcardForm').checkValidity()) {
    dispPcardModal(target)
    $(lookupPcardModal).hide()
  } else {
    document.getElementById('lookupPcardForm').reportValidity()
  }
})

document.getElementById('view_selected_pcard').addEventListener('click', ()=>{
  console.log('load selected pcard', selection)
  if (selection){
    dispPcardModal(selection)
  } else {
    createAlert('No Selection to Load!')
  }
})

mode = 'search'
document.getElementById('search_pcard').addEventListener('click', ()=>{
  console.log('lookup/create by cardnum')
  mode = 'search'
  document.getElementById('lookup_pcard_first_12').value = ''

  document.getElementById('lookup_pcard_first_12').removeAttribute('required')
  $(document.getElementById('lookup_pcard_first_12')).hide()

  document.getElementById('lookup_pcard_last_4').value = ''
  $(lookupPcardModal).show();
})
document.getElementById('add_new_pcard').addEventListener('click', ()=>{
  console.log('lookup/create by cardnum')
  mode = 'add'
  document.getElementById('lookup_pcard_first_12').value = ''

  document.getElementById('lookup_pcard_first_12').setAttribute('required', 'required')
  $(document.getElementById('lookup_pcard_first_12')).show()

  document.getElementById('lookup_pcard_last_4').value = ''
  $(lookupPcardModal).show();
})

// modal stuff:
let pcardModal = document.getElementById("pcardModal");
let lookupPcardModal = document.getElementById("lookupPcardModal");
let span = [...document.getElementsByClassName("close")];
console.log(span)
span.forEach(button =>{
  button.onclick = function() {
    $(pcardModal).hide()
    $(lookupPcardModal).hide()
  }
});

window.onclick = function(event) {
  if (event.target == pcardModal || event.target == lookupPcardModal) {
    $(pcardModal).hide()
    $(lookupPcardModal).hide()
  }
}

const pcardmodal_cardlist = document.getElementById('pcardmodal_cardlist')
const doc_emp_id = document.getElementById('emp_id')

async function dispPcardModal(instr=''){
  console.log('instr', instr)

  // document.getElementById('pcard_first_12').value = ''
  // document.getElementById('pcard_last_4').value = ''
  for (let i = pcardmodal_cardlist.length - 1; i >= 0; i--){
    pcardmodal_cardlist.remove(i);
  }

  document.getElementById('pcard_first_12').value = ''
  document.getElementById('pcard_last_4').value = ''
  document.getElementById('emp_uname').value =''
  document.getElementById('emp_lname').value =''
  document.getElementById('emp_fname').value = ''
  document.getElementById('pcard_cvv').value = ''
  document.getElementById('pcard_pin').value = ''
  document.getElementById('pcard_limit').value = ''
  document.getElementById('pcard_default_org_key').value = ''
  document.getElementById('pcard_default_obj_code').value = ''
  document.getElementById('pcard_notes').value = ''
  document.getElementById('pcard_issue_date').value = ''
  document.getElementById('pcard_expire_date').value = ''
  document.getElementById('pcard_enabled').value = ''
  document.getElementById('pcard_name_on_card').value = ''

  await ipcRenderer.invoke('load-single-pcard', instr).then(result =>{
    result.recordset.forEach(record=>{
      // console.log('record', record)
      option = document.createElement('option')
      option.value = JSON.stringify(record)
      option.text = record.pcard_first_12+'-'+record.pcard_last_4
      pcardmodal_cardlist.append(option)
    })

    if (result.recordset[0]){
      let arg = result.recordset[0]
      console.log('result', arg)
      $(pcardModal).show();
      document.getElementById('pcard_first_12').value = arg.pcard_first_12
      document.getElementById('pcard_last_4').value = arg.pcard_last_4
      document.getElementById('emp_uname').value = arg.emp_uname
      document.getElementById('emp_lname').value = arg.emp_lname
      document.getElementById('emp_fname').value = arg.emp_fname
      document.getElementById('pcard_cvv').value = arg.pcard_cvv
      document.getElementById('pcard_pin').value = arg.pcard_pin
      document.getElementById('pcard_limit').value = arg.pcard_limit
      document.getElementById('pcard_default_org_key').value = arg.pcard_default_org_key
      document.getElementById('pcard_default_obj_code').value = arg.pcard_default_obj_code
      document.getElementById('pcard_notes').value = arg.pcard_notes

      if (arg.pcard_issue_date){
        document.getElementById('pcard_issue_date').valueAsDate = new Date(arg.pcard_issue_date)
      }
      if (arg.pcard_expire_date){
        document.getElementById('pcard_expire_date').valueAsDate = new Date(arg.pcard_expire_date)
      }

      document.getElementById('pcard_enabled').value = arg.pcard_enabled

      document.getElementById('pcard_name_on_card').value = arg.pcard_name_on_card

      if (arg.emp_id && arg.emp_id != '') {
        doc_emp_id.value = arg.emp_id
        doc_emp_id.setAttribute('disabled', 'disabled')
        doc_emp_id.removeAttribute('required')
      } else {
        doc_emp_id.value = ''
        doc_emp_id.removeAttribute('disabled')
        doc_emp_id.setAttribute('required', 'required')
      }
    } else if (mode == 'search'){
      createAlert('Pcard Not Found!')
    } else if (instr.length == 16){
      createAlert('Info', 'Creating New Pcard!')

      option = document.createElement('option')
      option.value = instr.slice(0,12)+'-'+instr.slice(-4)
      option.text = instr.slice(0,12)+'-'+instr.slice(-4)
      pcardmodal_cardlist.append(option)

      document.getElementById('pcard_first_12').value = instr.slice(0,12)
      document.getElementById('pcard_last_4').value = instr.slice(-4)
      doc_emp_id.value = ''
      doc_emp_id.removeAttribute('disabled')
      doc_emp_id.setAttribute('required', 'required')
      $(pcardModal).show();
    } else {
      createAlert('No information for Pcard Found')
    }
  }).catch(err =>{
    createAlert('error', 'Load User PCARD Data Error: No PCARD Found for '+Permissions.emp_uname)
  })
}

document.getElementById('pcardmodal_cardlist').addEventListener('change', async event=>{
  await Promise.resolve(Permissions)
  arg = JSON.parse(event.target.value)
  console.log('selection', arg)
  document.getElementById('pcard_first_12').value = arg.pcard_first_12
  document.getElementById('pcard_last_4').value = arg.pcard_last_4
  document.getElementById('emp_uname').value = arg.emp_uname
  document.getElementById('emp_lname').value = arg.emp_lname
  document.getElementById('emp_fname').value = arg.emp_fname
  document.getElementById('pcard_cvv').value = arg.pcard_cvv
  document.getElementById('pcard_pin').value = arg.pcard_pin
  document.getElementById('pcard_limit').value = arg.pcard_limit
  document.getElementById('pcard_default_org_key').value = arg.pcard_default_org_key
  document.getElementById('pcard_default_obj_code').value = arg.pcard_default_obj_code
  document.getElementById('pcard_notes').value = arg.pcard_notes

  if (arg.pcard_issue_date){
    document.getElementById('pcard_issue_date').valueAsDate = new Date(arg.pcard_issue_date)
  }
  if (arg.pcard_expire_date){
    document.getElementById('pcard_expire_date').valueAsDate = new Date(arg.pcard_expire_date)
  }

  document.getElementById('pcard_enabled').value = arg.pcard_enabled

  document.getElementById('pcard_name_on_card').value = arg.pcard_name_on_card

  if (arg.emp_id && arg.emp_id != '') {
    doc_emp_id.value = arg.emp_id
    doc_emp_id.setAttribute('disabled', 'disabled')
    doc_emp_id.removeAttribute('required')
  } else {
    doc_emp_id.value = ''
    doc_emp_id.removeAttribute('disabled')
    doc_emp_id.setAttribute('required', 'required')
  }
})

document.getElementById('update-pcard').addEventListener('click', async ()=>{
  event.preventDefault()

  arg = {
    pcard_first_12:document.getElementById('pcard_first_12').value,
    pcard_last_4:document.getElementById('pcard_last_4').value,
    pcard_emp_id:document.getElementById('emp_id').value,
    emp_uname:document.getElementById('emp_uname').value,
    emp_lname:document.getElementById('emp_lname').value,
    emp_fname:document.getElementById('emp_fname').value,
    pcard_cvv:document.getElementById('pcard_cvv').value,
    pcard_pin:document.getElementById('pcard_pin').value,
    pcard_limit:parseInt(document.getElementById('pcard_limit').value),
    pcard_default_org_key:document.getElementById('pcard_default_org_key').value,
    pcard_default_obj_code:document.getElementById('pcard_default_obj_code').value,
    pcard_notes:document.getElementById('pcard_notes').value,
    pcard_issue_date:document.getElementById('pcard_issue_date').valueAsDate,
    pcard_expire_date:document.getElementById('pcard_expire_date').valueAsDate,
    pcard_name_on_card:document.getElementById('pcard_name_on_card').value,
    pcard_enabled:document.getElementById('pcard_enabled').value
  }

  console.log('update pcard', arg)
  if (document.getElementById('modifyPcardForm').checkValidity()) {
    console.log('form valid; submitting update query')
    result = await ipcRenderer.invoke('modify-single-pcard', arg)

    if (result) {
      console.log('success')
    } else {
      console.log('failure')
    }

    $(pcardModal).hide()
    reloadAllUsers()
    createAlert('success', 'Pcard Updated!')
  } else {
    console.log('Form invalid; Update impossible')
    // TODO: use these next two lines EVERYWHERE
    createAlert('danger', 'Form invalid; Update impossible')
    document.getElementById('modifyPcardForm').reportValidity()
  }

})

// on load everything else
reloadAllUsers()